import java.util.*;

public class BBTest {

    private static int nodes, depth;
    
    public static void search1(int n){
	nodes = depth = 0;
	BitSet C = new BitSet(n);
	BitSet P = new BitSet(n);
	P.set(0,n,true);
	search1(C,P);
    }
    
    private static void search1(BitSet C,BitSet P){
	nodes++;
	depth++;
	if (P.isEmpty()) System.out.println(C);
	//System.out.println("depth: "+ depth +" nodes: "+ nodes +" C: "+ C +" P: "+ P);
	while (P.cardinality() != 0){
	    int v = P.nextSetBit(0);
	    BitSet C_new = (BitSet)C.clone(); 
	    BitSet P_new = (BitSet)P.clone();
	    C_new.set(v,true);
	    P_new.set(v,false);
	    search1(C_new,P_new);
	    P.set(v,false);
	}
	depth--;
    }

     public static void search2(int n){
	 nodes = depth = 0;
	 BitSet C = new BitSet(n);
	 BitSet P = new BitSet(n);
	 P.set(0,n,true);
	 search2(C,P);
     }
    
    private static void search2(BitSet C,BitSet P){
	nodes++;
	depth++;
	if (P.isEmpty()) System.out.println(C);
	//System.out.println("depth: "+ depth +" nodes: "+ nodes +" C: "+ C +" P: "+ P);
	if (P.cardinality() != 0){
	    int v = P.nextSetBit(0);
	    BitSet C_new = (BitSet)C.clone(); 
	    BitSet P_new = (BitSet)P.clone();
	    C_new.set(v,true);
	    P_new.set(v,false);
	    search2(C_new,P_new);
	    C_new.set(v,false);
	    search2(C_new,P_new);
	}
	depth--;
    }
    
    public static void main(String[] args) {
	int n = Integer.parseInt(args[0]);
	int m = Integer.parseInt(args[1]);
	if (m == 1) search1(n);
	if (m == 2) search2(n);
    }
}
//
// search2 performs a bifurcating search for the power set with
// all sets occuring at depth n. At depth d we have 2^d states.
// In total 2^(n+1) - 1 states are visited
//
// search1 visits all sets of the power set as internal and as leaf nodes.
// At depth d we have Choose(n,d) states, and therefore a profile of
// search at depth gives us Pascal's triangle. In total 2^n states are visited
// 
// Consequently search1 is in the worst case twice as fast as search2
//
// search1 is used by Tomita and by San Segundo. search2 is used
// in the DFColSat constraint.
//
// This behaviour can also be modeled into a constraint program. search2 
// would correspond to zero/one variables whereas in search1 we have
// n variables each with domain {0..n} and constraints as follows
//
// IntegerVariable x[] = makeIntVarArray("x",n,0,n);
//   for (int i=0;i<n-1;i++)
//	  modl.addConstraint(or(gt(x[i],x[i+1]),and(eq(x[i],0),eq(x[i+1],0))));
//
// Patrick Prosser
// 16/02/2012
//
