import java.util.*;
import java.io.*;

public class KnapSack {

    private int capacity;
    private HashSet<Gem> gems;
    private HashSet<Gem> swag;
    private int value;
    private int bestValue;
    private int weight;
    private boolean trace;
    private int calls;

    public KnapSack(int capacity){
	this.capacity = capacity;
	gems = new HashSet<Gem>();
	swag = new HashSet<Gem>();
	value = weight = bestValue = 0;
	trace = false;
	calls = 0;
    }

    public HashSet<Gem> getSwag(){return swag;}
    public HashSet<Gem> getGems(){return gems;}
    public void setTrace(){trace = !trace;}

    private int getValue(HashSet<Gem> bag){
	int value = 0;
	for (Gem g : bag) value = value + g.getValue();
	return value;
    }

    private int getWeight(HashSet<Gem> bag){
	int weight = 0;
	for (Gem g : bag) weight = weight + g.getWeight();
	return weight;
    }

    public void add(Gem stone){gems.add(stone);}

    private Gem select(HashSet<Gem> bag){
	Iterator<Gem> it = bag.iterator();
	return it.next();
    }

    public String toString(){
	String s = "cap: "+ capacity + "  weight: "+ weight +"  value: "+ bestValue +"\n";
	s = s + "gems: ";
	for (Gem g : gems) s = s + g;
	s = s + "\nswag: ";
	for (Gem g : swag) s = s + g;
	s = s + "\ncalls: "+ calls;
	return s;
    }

    public void PS(HashSet<Gem> bag){
	HashSet<Gem> a = new HashSet<Gem>();
	HashSet<Gem> result = new HashSet<Gem>();
	bestValue = weight = calls = 0;
	PS(a,bag);
    }
    //
    // result is the set of sets that is the powerset of a
    //

    private void PS(HashSet<Gem> a,HashSet<Gem> b){
	calls++;
	if (getValue(a) > bestValue && getWeight(a) <= capacity){
	    swag = (HashSet<Gem>)a.clone(); bestValue = getValue(a); weight = getWeight(a);
	}
	if (trace) System.out.println(a +" "+ b +" "+ swag +" "+ weight +" "+ bestValue);
	if (!b.isEmpty() && getWeight(a) < capacity){
	    Gem x = select(b);
	    HashSet<Gem> aPrime = (HashSet<Gem>)a.clone();
	    HashSet<Gem> bPrime = (HashSet<Gem>)b.clone();
	    aPrime.add(x); bPrime.remove(x);
	    PS(aPrime,bPrime); PS(a,bPrime);
	}
    }
    //
    // take: take an element out of b, add it to a, then recurse
    // don't take: take an element out of b, do not add it to a, then recurse
    // Can be thought of as a binary tree with left branch "take" and
    // right branch "don't take"
    // This is a model for many algorithms, for example knapsack problem
    //

    public static void main(String[] args) throws Exception, IOException  {

	Scanner sc = new Scanner(new File(args[0]));
	int n = sc.nextInt();
	KnapSack KS = new KnapSack(n);

	while (sc.hasNext()) KS.add(new Gem(sc.nextInt(),sc.nextInt()));

	if (args.length > 1) KS.setTrace();

	System.out.println(KS);

	KS.PS(KS.getGems());

	System.out.println(KS);
    }
}
